/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/

#include <stdio.h>
#include <stdlib.h>


typedef struct _elf_header {
_u8	ident[4];								//0x00 - 4 bytes		0x7f,"ELF"  (ELF file identifier)
_u8	file_class;								//0x04 - 1 byte		File Class: 1=32bit, 2=64bit objects
_u8	data_encoding;						//0x05 - 1 byte		Data Encoding: 1=ELFDATA2LSB, 2=ELFDATA2MSB
_u8	hdr_vers;								//0x06 - 1 byte		ELF header version (must be 1)
_u8	zero[9];								//0x07 - 9 bytes		zeroised
_u16	elf_type;								//0x10 - 2 bytes		ELF type: 0=NONE, 1=REL, 2=EXEC, 3=SHARED, 4=CORE
_u16	processor	;							//0x12 - 2 bytes		Processor: 8=MIPS R3000
_u32	version;								//0x14 - 4 bytes		Version: 1=current
_u32	entry_point_add;					//0x18 - 4 bytes		Entry point address
_u32	start_prg_hdr;						//0x1C - 4 bytes		Start of program headers (offset from file start)
_u32	start_sct_hdr;						//0x20 - 4 bytes		Start of section headers (offset from file start)
_u32	proce_spec_flag;					//0x24 - 4 bytes		Processor specific flags = 0x20924001 noreorder, mips
_u16	elf_hed_size;							//0x28 - 2 bytes		ELF header size (0x34 = 52 bytes)
_u16	prg_hdr_entry_size;				//0x2A - 2 bytes		Program headers entry size 
_u16	num_of_prg_hdr;					//0x2C - 2 bytes		Number of program headers
_u16	sec_hdr_entry_size;				//0x3E - 2 bytes		Section headers entry size
_u16	num_of_sec_hdr;					//0x30 - 2 bytes		Number of section headers
_u16	sec_hdr_stringa_index;			//0x32 - 2 bytes		Section header stringtable index	
}
 elf_header;

/*
Program Header (PHR):
=====================

Offset located in EHR 0x1C, size specified by EHR 0x2A and the number of PHR's
specified in EHR 0x2C.

*/

typedef struct _elf_program_header {
_u32	segment_type;						//0x00 - 4 bytes		Segment type: 
													//								0=Inactive
													//								1=Load the segment into memory, no. of bytes specified by 0x10 and 0x14
													//								2=Dynamic linking
													//								3=Interpreter. The array element must specify a path name
													//								4=Note. The array element must specify the location and size of aux. info
													//								5=reserved
													//								6=The array element must specify location and size of the program header table.
													//								0x70000000 - 0x7fffffff processor specific semantics
_u32	offs_from_start_to_prg_seg;		//0x04 - 4 bytes		Offset from file start to program segment.
_u32	virtual_add_of_segment;			//0x08 - 4 bytes		Virtual address of the segment
_u32	physic_add_of_segment;			//0x0C - 4 bytes		Physical address of the segment
_u32	num_of_bytes_in_file_ima;			//0x10 - 4 bytes		Number of bytes in the file image of the segment
_u32	num_of_bytes_in_mem_ima;		//0x14 - 4 bytes		Number of bytes in the memory image of the segment
_u32	flags;										//0x18 - 4 bytes		Flags for segment
_u32	alignement;								//0x1C - 4 bytes		Alignment. The address of 0x08 and 0x0C must fit this alignment. 0=no alignment

} elf_program_header;

/*
Section Header (SHR):
=====================

Offset located in EHR 0x20, size specified by EHR 0x30 and number of SHR's specified in EHR 0x32.

*/


typedef struct _elf_section_header {
_u32	numero_index_sect;							//0x00 - 4 bytes		No. to the index of the Section header stringtable index
_u32	type;													//0x04 - 4 bytes		Type:
																	//								0=Inactive
																	//								1=PROGBITS
																	//								2=SYMTAB symbol table
																	//								3=STRTAB string table
																	//								4=RELA relocation entries
																	//								5=HASH hash table
																	//								6=DYNAMIC dynamic linking information
																	//								7=NOTE
																	//								8=NOBITS 
																	//								9=REL relocation entries
																	//								10=SHLIB
																	//								0x70000000=LOPROC processor specifc
																	//								0x7fffffff=HIPROC
																	//								0x80000000=LOUSER lower bound
																	//								0xffffffff=HIUSER upper bound
_u32	sect_flag;											//0x08 - 4 bytes		Section Flags:  (1 bit, you may combine them like 3 = alloc & write permission)
																	//								1=Write section contains data the is be writeable during execution.
																	//								2=Alloc section occupies memory during execution
																	//								4=Exec section contains executable instructions
																	//								0xf0000000=Mask bits processor-specific
_u32	sect_start_add;									//0x0C - 4 bytes		Section start address
_u32	offset_from_start;								//0x10 - 4 bytes		Offset from start of file to section
_u32	size;													//0x14 - 4 bytes		Size of section
_u32	sect_hdr_table_index;						//0x18 - 4 bytes		Section header table index link
_u32	info;													//0x1C - 4 bytes		Info
_u32	alignement;										//0x20 - 4 bytes		Alignment. The adress of 0x0C must fit this alignment. 0=no alignment.
_u32	fixed_size_entry;								//0x24 - 4 bytes		Fixed size entries.
}elf_section_header;


extern elf_header ico_elf_hdr;
extern elf_program_header ico_elf_prg_hdr[30];
extern elf_section_header ico_elf_sec_hdr[30];




