/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdio.h>
#include <stdlib.h>

#include "nSX2.h"
#include "r5900_main.h"
#include "r5900_main_table.h"
 
#include "muGUI/muGUI.h"

/*
Globals variables 
Used for decoding the opcode
Each vars is a field of the opcode
*/
_u8		rt;
_u8		rs;
_u8		rd;
_u8		base;
_u8		sa;
_u64	imm;
_u64	offset;
_u64	instr_index;
_u8		sel;
_u8		ft;
_u8		fd;
_u8		fs;
_u8		cc;
_u8		ccb;

char debugTempText[100];
char debugTempOpcodeText[100];
/*
Print the the current opcode
*/
void PrintDebugOpcode() {
	printf("==>%8lx\n",MipsCpu.opcode);
	
	switch ( ((MipsCpu.opcode >> 26) & 0x3F) ) {
	
		case 0 	: 	printf("r5900:dbg_opcode_special: %s \n", debug_special_opc[(MipsCpu.opcode  & 0x3F)]); break;
		case 1	: 	printf("r5900:dbg_opcode_regimm: %s \n", debug_regimm_opc[(MipsCpu.opcode  & 0x1F0000)>>16]); break;
		case 16 : 	printf("r5900:dbg_opcode_cop0: %s \n", debug_cop0_opc[(MipsCpu.opcode&0x3E00000)>>21]);
				  	printf("r5900:dbg_opcode_cop0c0: %s \n", debug_cop0c0_opc[(MipsCpu.opcode&0x3F)]);
					break;
		case 17 : {
					switch( (MipsCpu.opcode&0x3E00000)>>21) {
						
						case 8:  printf("r5900:debug_cop1_BC1_opc: %s \n",	debug_cop1_bc1_opc[(MipsCpu.opcode>>16)&0x3] );break;					
						case 16: printf("r5900:debug_cop1_S_opc: %s \n", 	debug_cop1_s_opc[MipsCpu.opcode&0x3F] 	);break;					
						case 20: printf("r5900:debug_cop1_W_opc: %s \n", 	debug_cop1_W_L_opc[MipsCpu.opcode&0x3F] );break;					

						default: printf("r5900:dbg_opcode_cop1: %s \n", debug_cop1_opc[(MipsCpu.opcode&0x3E00000)>>21]);break;					
					}
					
					break;
				}	
	
		case 28 : printf("r5900:dbg_opcode_special2: %s \n", debug_special2_opc[(MipsCpu.opcode  & 0x3F)] );
	
		default : printf("r5900:dbg_opcode: %s \n", debug_normal_opc[((MipsCpu.opcode >> 26) & 0x3F)]); break;
	
	}

}

/*
Print the CPU Registers
Only the main registers and only the low part
*/

void MipsCpuPrintReg( void ) {
	_u8	i;

	for ( i = 0 ; i < 11; i++ ) {
	
		printf("%s : %16.16llx  | %s : %16.16llx | %s :%16.16llx  | \n",  
		debug_r5900_reg[ (i*3)+0 ], MipsCpu.GPR[(i*3)+0].lo ,debug_r5900_reg[(i*3)+1], MipsCpu.GPR[(i*3)+1].lo ,  
		debug_r5900_reg[(i*3)+2], MipsCpu.GPR[(i*3)+2].lo   );
	}
	
	printf("%s : %16.16llx  |      \n",  
	debug_r5900_reg[CPU_REG_LO], MipsCpu.GPR[CPU_REG_LO].lo 	  );

}


/*
Print the FPU Registers
Only the float format ( emu format ) not the real format ( cpu format )
*/
void MipsCpuFPR1PrintReg( void ) {
	_u8	i;

	for ( i = 0 ; i < 10; i++ ) {
	
		printf("%4s : %12.4f  | %4s : %12.4f | %4s :%12.4f  | \n",  
		debug_r5900_cop1_reg[ (i*3)+0 ], MipsCpu.FPR1_float[(i*3)+0] ,
		debug_r5900_cop1_reg[(i*3)+1], MipsCpu.FPR1_float[(i*3)+1] ,
		debug_r5900_cop1_reg[(i*3)+2], MipsCpu.FPR1_float[(i*3)+2]    );
	}
	
	printf("%4s : %12.4f  | %4s : %12.4f | \n",  
	debug_r5900_cop1_reg[ 30 ], MipsCpu.FPR1_float[30] ,
	debug_r5900_cop1_reg[ 31 ], MipsCpu.FPR1_float[31] );
	
}


void MipsCpuPrintRegtoOutput( void ) {
	_u8	i=0;
	
	for ( i = 0 ; i < 34 ; i ++ ) {
		sprintf(debugTempOpcodeText,"%s : %16.16llx",debug_r5900_reg[ i ], MipsCpu.GPR[i].lo );
		PrintToRegistersOutput ( debugTempOpcodeText );
	}

}


/*
Main CPU functions
*/

void MipsUpdateCpu ( ) {
	
	
		
	MipsCpu.Nb_instructions = MipsCpu.Nb_instructions ++;			
	
	MipsFillOpcode(MipsCpu.PC);										//Read the memory and store the val to MipsCpu.opcode
	
	if ( MipsCpu.opcode != 0 ) 	{									//if the opcode is different of NOP
		normal_opc[((MipsCpu.opcode >> 26) & 0x3F)]();				//Execute the current opcode
	}
	else {															//else
		if ( debug_op == 1 ) {										//the opcode is NOP
			sprintf(debugTempText,"%8.8llx: nop", MipsCpu.PC);				
			
			
		}
		MipsCpu.PC = MipsCpu.PC + 4;		 	
	}
	
	if ( debug_op == 1 ) {
		PrintToOpcodeOutput ( debugTempText );
		MipsCpuPrintRegtoOutput();
	}
	
}

 

/*****************************************************************************************************************/


void SPECIAL() {	special_opc[(MipsCpu.opcode  & 0x3F)]();	}


void REGIMM() {		regimm_opc[(MipsCpu.opcode  & 0x1F0000)>>16]();		}

/*
Jump
To branch within the current 256 MB-aligned region
MIPS32
*/
void J() {
	_u64 	target_branch;

	Extract_instr_index();
	
	target_branch = instr_index  << 2;

	if ( debug_op == 1 ) 	{
		sprintf(debugTempText,"%8.8llx: j %8llx", MipsCpu.PC, target_branch );
		PrintToOpcodeOutput ( debugTempText );
	}
	
	
	MipsCpu.PC = MipsCpu.PC + 4;
	MipsUpdateCpu();
	
	MipsCpu.PC = ( MipsCpu.PC & 0xFFFFFFFF00000000 ) | target_branch;
}

/*
Jump and Link
To execute a procedure call within the current 256 MB-aligned region
MIPS32
*/
void     JAL  () {

	_u64 	target_branch;

	Extract_instr_index();
	MipsCpu.GPR[31].lo =  MipsCpu.PC + 8;
	
	target_branch = instr_index  << 2;

	if ( debug_op == 1 ) 	{
		sprintf(debugTempText,"%8.8llx: jal %8llx", MipsCpu.PC, target_branch );
		PrintToOpcodeOutput ( debugTempText );
	}
	
	
	MipsCpu.PC = MipsCpu.PC + 4;
	MipsUpdateCpu();
	
	MipsCpu.PC = ( MipsCpu.PC & 0xFFFFFFFF00000000 ) | target_branch;
}

/*
Branch on Equal
To compare GPRs then do a PC-relative conditional branch
MIPS32
*/
void     BEQ () {
	_u64	target_branch;
	
	Extract_rs();
	Extract_rt();
	Extract_offset();
	
	offset = offset << 2;
	sign_extendedh(offset);
	target_branch = MipsCpu.PC + 4 +  offset ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: beq %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[rs],debug_r5900_reg[rt],target_branch );

	if ( MipsCpu.GPR[rs].lo == MipsCpu.GPR[rt].lo ) {
			if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
			MipsCpu.PC = MipsCpu.PC + 4;
			MipsUpdateCpu();
			MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 4;
} 

/*
Branch on Not Equal
To compare GPRs then do a PC-relative conditional branch
MIPS32
*/
void     BNE () {

	_u64	target_branch;
	
	Extract_rs();
	Extract_rt();
	Extract_offset();
	
	offset = offset << 2;
	sign_extendedh(offset);
	target_branch = MipsCpu.PC + 4 +  offset ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bne %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[rs],debug_r5900_reg[rt],target_branch );

	if ( MipsCpu.GPR[rs].lo != MipsCpu.GPR[rt].lo ) {
			if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
			MipsCpu.PC = MipsCpu.PC + 4;
			MipsUpdateCpu();
			MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 4;
	

}

/*
Branch on Less Than or Equal to Zero
To test a GPR then do a PC-relative conditional branch
MIPS32
*/
void     BLEZ () {
	_u64	target_branch;
	
	Extract_rs();
	Extract_offset();
	
	offset = offset << 2;
	sign_extendedh(offset);
	target_branch = MipsCpu.PC + 4 +  offset ;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: blez %s,%4llx", MipsCpu.PC, debug_r5900_reg[rs], target_branch );

	if ( (_s64)(MipsCpu.GPR[rs].lo) <= 0 ) {
			if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
			MipsCpu.PC = MipsCpu.PC + 4;
			MipsUpdateCpu();
			MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Branch on Greater Than Zero
To test a GPR then do a PC-relative conditional branch
MIPS32
*/
void     BGTZ () {
	_u64	target_branch;
	
	Extract_rs();
	Extract_offset();
	
	offset = offset << 2;
	sign_extendedh(offset);
	target_branch = MipsCpu.PC + 4 +  offset ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bgtz %s,%4llx", MipsCpu.PC, debug_r5900_reg[rs], target_branch );

	

	if ( (_s64)( MipsCpu.GPR[rs].lo ) > 0 ) {
			if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
			MipsCpu.PC = MipsCpu.PC + 4;
			MipsUpdateCpu();
			MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Add Immediate Word
To add a constant to a 32-bit integer. If overflow occurs, then trap.
MIPS32
*/
void    ADDI   () {
	_u64 temp;
	
	Extract_rt(); 
	Extract_rs();
	Extract_imm();
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: addi %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[rt],debug_r5900_reg[rs],imm );
	
	sign_extendedh(imm);
	temp = MipsCpu.GPR[rs].lo + imm;
	sign_extended(temp);
	MipsCpu.GPR[rt].lo = temp;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Add Immediate Unsigned Word
To add a constant to a 32-bit integer
MIPS32
*/
void    ADDIU () {
	_u64 temp;
	
	Extract_rt(); 
	Extract_rs();
	Extract_imm();
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: addiu %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[rt],debug_r5900_reg[rs],imm );

	
	sign_extendedh(imm);
	temp = MipsCpu.GPR[rs].lo + imm;
	sign_extended(temp);
	MipsCpu.GPR[rt].lo = temp;


	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Set on Less Than Immediate
To record the result of a less-than comparison with a constant
MIPS32
*/
void    SLTI() {
	
	Extract_rt(); 
	Extract_rs();
	Extract_imm();

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: slti %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[rt],debug_r5900_reg[rs],imm );
	sign_extendedh(imm);
	
	if ( (_s64)MipsCpu.GPR[rs].lo < (_s64) imm ) 	MipsCpu.GPR[rt].lo = 1;
	else MipsCpu.GPR[rt].lo = 0;


	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Set on Less Than Immediate Unsigned
To record the result of an unsigned less-than comparison with a constant
MIPS32
*/
void    SLTIU() {
	Extract_rs();
	Extract_rt();
	Extract_imm();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sltiu %s,%s,%8llx", MipsCpu.PC, debug_r5900_reg[rd], debug_r5900_reg[rs], imm);
	sign_extendedh(imm);
 
    if  ( 	MipsCpu.GPR[rs].lo < imm ) 	MipsCpu.GPR[rt].lo = 1; 
    else 	MipsCpu.GPR[rt].lo = 0; 
	
	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
And Immediate
To do a bitwise logical AND with a constant
MIPS32
*/
void    ANDI() {

	
	Extract_rt(); 
	Extract_rs();
	Extract_imm();
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: andi %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[rt],debug_r5900_reg[rs],imm );

	MipsCpu.GPR[rt].lo = MipsCpu.GPR[rs].lo & imm;


	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Or Immediate
To do a bitwise logical OR with a constant
MIPS32
*/
void    ORI () {

	
	Extract_rt(); 
	Extract_rs();
	Extract_imm();
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: ori	%s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[rt],debug_r5900_reg[rs],imm );

	MipsCpu.GPR[rt].lo = MipsCpu.GPR[rs].lo | imm;


	MipsCpu.PC = MipsCpu.PC + 4;
}


/*
Exclusive OR Immediate
To do a bitwise logical Exclusive OR with a constant
MIPS32
*/
void    XORI () {
	Extract_rt(); 
	Extract_rs();
	Extract_imm();
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: xori %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[rt],debug_r5900_reg[rs],imm );

	MipsCpu.GPR[rt].lo = MipsCpu.GPR[rs].lo ^ imm;


	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Load Upper Immediate
To load a constant into the upper half of a word
MIPS32
*/
void    LUI  () {
	Extract_rt(); 
	Extract_imm();
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: lui %s,%4llx", MipsCpu.PC, debug_r5900_reg[rt],imm );
	
	imm = imm << 16;
	sign_extended(imm);
	MipsCpu.GPR[rt].lo = imm ;
	MipsCpu.PC = MipsCpu.PC + 4;
	

}


void    COP0()	{	cop0_opc[(MipsCpu.opcode&0x3E00000)>>21]();}


void    COP1()	{	cop1_opc[(MipsCpu.opcode&0x3E00000)>>21]();}


void    COP2()	{	MipsCpu.halt = 1;}
void    COP1X()	{	MipsCpu.halt = 1;}


/*
Branch on Equal Likely
To compare GPRs then do a PC-relative conditional branch; execute the delay slot only if the branch is taken.
MIPS32
*/
void     BEQL() {
	_u64	target_branch;
	
	Extract_rs();
	Extract_rt();
	Extract_offset();
	
	offset = offset << 2;
	sign_extendedh(offset);
	target_branch = MipsCpu.PC + 4 +  offset ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: beql %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[rs],debug_r5900_reg[rt],target_branch );

	if ( MipsCpu.GPR[rs].lo == MipsCpu.GPR[rt].lo ) {
			if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
			MipsCpu.PC = MipsCpu.PC + 4;
			MipsUpdateCpu();
			MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8;
}

/*
Branch on Not Equal Likely
To compare GPRs then do a PC-relative conditional branch; execute the delay slot only if the branch is taken.
MIPS32
*/
void     BNEL() {
	_u64	target_branch;
	
	Extract_rs();
	Extract_rt();
	Extract_offset();
	
	offset = offset << 2;
	sign_extendedh(offset);
	target_branch = MipsCpu.PC + 4 +  offset ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bnel %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[rs],debug_r5900_reg[rt],target_branch );

	if ( MipsCpu.GPR[rs].lo != MipsCpu.GPR[rt].lo ) {
			if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
			MipsCpu.PC = MipsCpu.PC + 4;
			MipsUpdateCpu();
			MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8;
}


/*
Branch on Less Than or Equal to Zero Likely
To test a GPR then do a PC-relative conditional branch; execute the delay slot only if the branch is taken.
MIPS32
*/
void    BLEZL() {		
	_u64	target_branch;
	
	Extract_rs();

	Extract_offset();
	
	offset = offset << 2;
	sign_extendedh(offset);
	target_branch = MipsCpu.PC + 4 +  offset ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: blezl %s,%4llx", MipsCpu.PC,debug_r5900_reg[rs],target_branch );

	if ( (_s64)MipsCpu.GPR[rs].lo <= 0 ) {
			if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
			MipsCpu.PC = MipsCpu.PC + 4;
			MipsUpdateCpu();
			MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8;
}

void    BGTZL() {	MipsCpu.halt = 1;}
void    DADDI() {	MipsCpu.halt = 1;}

/*
Doubleword Add Immediate Unsigned
To add a constant to a 64-bit integer
MIPS64
*/
void    DADDIU    () {
	Extract_rt(); 
	Extract_rs();
	Extract_imm();
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: daddiu %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[rt],debug_r5900_reg[rs],imm );
	
	sign_extendedh(imm);
	MipsCpu.GPR[rt].lo = MipsCpu.GPR[rs].lo + imm;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Load Doubleword Left
To load the most-significant part of a doubleword from an unaligned memory address
MIPS64
*/
void    LDL  () {
	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 )		sprintf(debugTempText,"%8.8llx: ldl %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	
		
	MipsMemoryRead ( &MipsCpu.GPR[rt] , DATA_DWH ,  MipsCpu.GPR[base].lo , offset ) ;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Load Doubleword Right
To load the least-significant part of a doubleword from an unaligned memory address
MIPS64
*/
void    LDR   () {

	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 )		sprintf(debugTempText,"%8.8llx: ldr %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	
		
	MipsMemoryRead ( &MipsCpu.GPR[rt] , DATA_DW ,  MipsCpu.GPR[base].lo , offset ) ;

	MipsCpu.PC = MipsCpu.PC + 4;
	

}


void    SPECIAL2()	{	special2_opc[(MipsCpu.opcode  & 0x3F)](); }

 
void    JALX()		{	MipsCpu.halt = 1;}

/*

A MDMX Coprocessor Unusable exception occurs when an attempt is made to execute a MDMX instruction if either:
-No coprocessor implementing MDMX is attached to the processor
-The attached MDMX coprocessor has not been enabled by the MD bit in the Status register.

Cause Register ExcCode Value
MDMX

Additional State Saved
None.

Exception Vector Used
General exception vector, offset 0x180

*/
void    MDMX()		{	
	
	printf("mdmx\n");
	//MipsCpu.halt = 1;
	MipsCpu.PC = MipsCpu.PC + 4;
}

void    LQ()		{	
	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lq %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );
	
	sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[rt] , DATA_QUAD ,  MipsCpu.GPR[base].lo , offset ) ;
	sign_extended( MipsCpu.GPR[rt].lo );

	MipsCpu.PC = MipsCpu.PC + 4;
}


void    NI()		{	MipsCpu.halt = 1;}

/*
Load Byte
To load a byte from memory as a signed value
MIPS32
*/
void    LB     () {
	
	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lb %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[rt] , DATA_B ,  MipsCpu.GPR[base].lo , offset ) ;
	sign_extendedb( MipsCpu.GPR[rt].lo );

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Load Halfword
To load a halfword from memory as a signed value
MIPS32
*/
void    LH    () {
	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lh %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[rt] , DATA_HW ,  MipsCpu.GPR[base].lo , offset ) ;
	sign_extendedh( MipsCpu.GPR[rt].lo );

	MipsCpu.PC = MipsCpu.PC + 4;
}


/*
Load Word Left
To load the most-significant part of a word as a signed value from an unaligned memory address
MIPS32
*/
void    LWL () { 	

	_u128	temp128;
	_u64	temp64;

	Extract_base();
	Extract_rt();
	Extract_offset();
	

	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lwl %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	MipsMemoryRead ( &temp128 , DATA_W ,  MipsCpu.GPR[base].lo , offset ) ;
	
	//printf("LWL : %8.8llx =>", MipsCpu.GPR[rt].lo );
	
	temp64 =  temp128.lo   & 0xFFFF0000;
	
	MipsCpu.GPR[rt].lo = ( MipsCpu.GPR[rt].lo & 0x0000FFFF )  | temp64;
	
	sign_extended( MipsCpu.GPR[rt].lo );
	
	//printf(" %8.8llx \n", MipsCpu.GPR[rt].lo );

	MipsCpu.PC = MipsCpu.PC + 4; 
	//MipsCpu.halt = 1;
}

/*
Load Word
To load a word from memory as a signed value
MIPS32
*/
void    LW   () {
	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lw %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[rt] , DATA_W ,  MipsCpu.GPR[base].lo , offset ) ;
	sign_extended( MipsCpu.GPR[rt].lo );

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Load Byte Unsigned
To load a byte from memory as an unsigned value
MIPS32
*/
void    LBU () {
	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lbu %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[rt] , DATA_B ,  MipsCpu.GPR[base].lo , offset ) ;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Load Halfword Unsigned
To load a halfword from memory as an unsigned value
MIPS32
*/
void    LHU () {
	Extract_base();
	Extract_rt();
	Extract_offset();
	
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lhu %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);

	MipsMemoryRead ( &MipsCpu.GPR[rt] , DATA_HW ,  MipsCpu.GPR[base].lo , offset ) ;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Load Word Right
To load the least-significant part of a word from an unaligned memory address as a signed value
MIPS32
*/
void    LWR  () {		

	_u128	temp128;
	_u64	temp64;

	Extract_base();
	Extract_rt();
	Extract_offset();
	

	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lwr %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	MipsMemoryRead ( &temp128 , DATA_W ,  MipsCpu.GPR[base].lo , offset ) ;
	
	//printf("LWR : %8.8llx =>", MipsCpu.GPR[rt].lo );
	
	temp64 = ( temp128.lo  ) & 0xFFFF;
	
	MipsCpu.GPR[rt].lo = ( MipsCpu.GPR[rt].lo & 0xFFFF0000 )  | temp64;
	
	sign_extended( MipsCpu.GPR[rt].lo );
	
	//printf(" %8.8llx \n", MipsCpu.GPR[rt].lo );

	MipsCpu.PC = MipsCpu.PC + 4; 
	//MipsCpu.halt = 1;
}

/*
Load Word Unsigned
To load a word from memory as an unsigned value
MIPS64
*/
void    LWU   () {
	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lwu %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[rt] , DATA_W ,  MipsCpu.GPR[base].lo , offset ) ;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Store Byte
To store a byte to memory
MIPS32
*/
void    SB     () {
	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sb %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	MipsMemoryWrite ( MipsCpu.GPR[rt] , DATA_B ,  MipsCpu.GPR[base].lo , offset ) ;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Store Halfword
To store a halfword to memory
MIPS32
*/
void    SH    () {
	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sh %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	MipsMemoryWrite ( MipsCpu.GPR[rt] , DATA_HW ,  MipsCpu.GPR[base].lo , offset ) ;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Store Word Left
To store the most-significant part of a word to an unaligned memory address
MIPS32
*/
void    SWL () {		
	_u128	temp128;
	_u64	temp64;

	Extract_base();
	Extract_rt();
	Extract_offset();
	

	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: swl %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	
	temp64 = MipsCpu.GPR[rt].lo;
	temp64 = temp64 & 0xFFFF0000;
	temp128.lo = temp64;

	MipsMemoryWrite ( temp128 , DATA_W ,  MipsCpu.GPR[base].lo , offset ) ;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Store Word
To store a word to memory
MIPS32
*/
void    SW   () {
	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sd %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );
	
	sign_extendedh(offset);
	MipsMemoryWrite ( MipsCpu.GPR[rt] , DATA_W ,  MipsCpu.GPR[base].lo , offset ) ;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Store Doubleword Left
To store the most-significant part of a doubleword to an unaligned memory address
MIPS64
*/
void    SDL  () {
	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sdl %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	MipsMemoryWrite ( MipsCpu.GPR[rt] , DATA_DWH ,  MipsCpu.GPR[base].lo , offset ) ;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Store Doubleword Right
To store the least-significant part of a doubleword to an unaligned memory address
MIPS64
*/
void    SDR  () {
	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sdr %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	MipsMemoryWrite ( MipsCpu.GPR[rt] , DATA_DW ,  MipsCpu.GPR[base].lo , offset ) ;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Store Word Right
To store the least-significant part of a word to an unaligned memory address
MIPS32
*/
void    SWR  () {		
	_u128	temp128;
	_u64	temp64;

	Extract_base();
	Extract_rt();
	Extract_offset();
	

	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: swl %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	
	temp64 = MipsCpu.GPR[rt].lo;
	temp64 = temp64 & 0xFFFF;
	temp128.lo = temp64;

	MipsMemoryWrite ( temp128 , DATA_W ,  MipsCpu.GPR[base].lo , offset ) ;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Perform Cache Operation
To perform the cache operation specified by op.
MIPS32
*/
void    CACHE( ) {

	sprintf(debugTempText,"%8.8llx: CACHE", MipsCpu.PC);
	MipsCpu.PC = 	MipsCpu.PC + 4 ;
}


void    LL   () {	MipsCpu.halt = 1;}

/*
Load Word to Floating Point LWC1
To load a word from memory to an FPR
MIPS32
*/
void    LWC1 () {		

	_u128	temp;
	
	Extract_base();
	Extract_ft();
	Extract_offset();
	

	
	
	
	if ( debug_op == 1 ) 		{
		sprintf(debugTempText,"%8.8llx: lwc1 %s,%4llx(%s)", MipsCpu.PC, debug_r5900_cop1_reg[ft],offset,debug_r5900_reg[base] );
		
		sprintf(debugTempText,"%8.8llx: %s = %8.8llx ", MipsCpu.PC,debug_r5900_cop1_reg[ft],MipsCpu.FPR1[ft].lo);
	}
	
	sign_extendedh(offset);
	MipsMemoryRead ( &temp , DATA_W ,  MipsCpu.GPR[base].lo , offset ) ;
	
	//sign_extended(temp.lo);
	
	MipsCpu.FPR1[ft].lo = temp.lo;
	MipsCpu.FPR1_float[ft] = *(MipsCpu.FPR1_p_float[ft]);			//Make the conversion to the Emu Val
	
	//printf("%8llx ==> LWC1 :%s  = %16.5f \n", MipsCpu.PC, debug_r5900_cop1_reg[ft] ,MipsCpu.FPR1_float[ft]);
	
	//MipsCpu.halt = 1;
	MipsCpu.PC = MipsCpu.PC + 4;
	
	
}


void    LWC2 () {	MipsCpu.halt = 1;}
void    PREF () {	MipsCpu.halt = 1;}
void    LLD  () {	MipsCpu.halt = 1;}
void    LDC1 () {	MipsCpu.halt = 1;}
void    LDC2 () {	MipsCpu.halt = 1;}

/*
Load Doubleword
To load a doubleword from memory
MIPS64
*/
void    LD   () {
	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: ld %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );
	
	sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[rt] , DATA_DW ,  MipsCpu.GPR[base].lo , offset ) ;
	//sign_extended( MipsCpu.GPR[rt].lo );

	MipsCpu.PC = MipsCpu.PC + 4;
	

}

void    SC     () {	MipsCpu.halt = 1;}


/*
Store Word from Floating Point
To store a word from an FPR to memory
MIPS32
*/
void    SWC1    () {
	Extract_base();
	Extract_ft();
	Extract_offset();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sdwc1 fp%d,%4llx(%s)", MipsCpu.PC, ft,offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	
	MipsMemoryWrite ( MipsCpu.FPR1[ft] , DATA_W ,  MipsCpu.GPR[base].lo , offset ) ;

	MipsCpu.PC = MipsCpu.PC + 4;
}


void    SWC2  () {	MipsCpu.halt = 1;}



void SQ() {

	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sq %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	MipsMemoryWrite ( MipsCpu.GPR[rt] , DATA_QUAD ,  MipsCpu.GPR[base].lo , offset ) ;
	MipsCpu.PC = MipsCpu.PC + 4;
}

void    SCD  () {	MipsCpu.halt = 1;}
void    SDC1 () {	MipsCpu.halt = 1;}
void    SDC2 () {	MipsCpu.halt = 1;}

/*
Store Doubleword
To store a doubleword to memory
MIPS64
*/
void    SD() {
	Extract_base();
	Extract_rt();
	Extract_offset();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sd %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[rt],offset,debug_r5900_reg[base] );

	sign_extendedh(offset);
	MipsMemoryWrite ( MipsCpu.GPR[rt] , DATA_DW ,  MipsCpu.GPR[base].lo , offset ) ;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Shift Word Left Logical
To left-shift a word by a fixed number of bits
MIPS32
*/
void   SLL() {
	_u64 temp;
	
	Extract_rt();
	Extract_rd();
	Extract_sa();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sll %s,%s,%d", MipsCpu.PC, debug_r5900_reg[rd], debug_r5900_reg[rt], sa );
	
	temp = ( MipsCpu.GPR[rt].lo & 0xFFFFFFFF ) <<  sa;	
	sign_extended(temp);
	MipsCpu.GPR[rd].lo = temp;
	
	MipsCpu.PC = MipsCpu.PC + 4;
}


void   MOVCI() {	MipsCpu.halt = 1;}

/*
Shift Word Right Logical
To execute a logical right-shift of a word by a fixed number of bits
MIPS32
*/
void   SRL   () {
	_u64 temp;
	
	Extract_rt();
	Extract_rd();
	Extract_sa();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: srl %s,%s,%d", MipsCpu.PC, debug_r5900_reg[rd], debug_r5900_reg[rt], sa );
	
	temp = ( MipsCpu.GPR[rt].lo & 0xFFFFFFFF )  >>  sa;	
	sign_extended(temp);
	MipsCpu.GPR[rd].lo = temp;
	
	MipsCpu.PC = MipsCpu.PC + 4;
}


/*
Shift Word Right Arithmetic
To execute an arithmetic right-shift of a word by a fixed number of bits
MIPS32
*/
void   SRA  () {
	_u64 temp;
	_u64 boucle;
	
	Extract_rt();
	Extract_rd();
	Extract_sa();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sra %s,%s,%d", MipsCpu.PC, debug_r5900_reg[rd], debug_r5900_reg[rt], sa );
	
	if (MipsCpu.GPR[rt].lo & 0x0000000080000000LL) {
		//printf("SRA %d: %8.8llx => ",sa,MipsCpu.GPR[rt].lo);
		temp = MipsCpu.GPR[rt].lo & 0xFFFFFFFF;
		for ( boucle = 0 ; boucle < sa ; boucle++ ) temp = ( temp >> 1 ) | 0x80000000 ;
		//printf("%8.8llx\n",temp);
	} else 	temp =  MipsCpu.GPR[rt].lo >>  sa  ; 
	
	
	sign_extended(temp);
	MipsCpu.GPR[rd].lo = temp;
	
	MipsCpu.PC = MipsCpu.PC + 4;
	


}

/*
Shift Word Left Logical Variable
To left-shift a word by a variable number of bits
MIPS32
*/
 
void   SLLV () {		
	_u64 temp;
	
	Extract_rt();
	Extract_rd();
	Extract_rs();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sllv %s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd], debug_r5900_reg[rt], debug_r5900_reg[rs] );
	
	temp = ( MipsCpu.GPR[rt].lo & 0xFFFFFFFF ) <<  ( MipsCpu.GPR[rs].lo & 0x1F);	
	sign_extended(temp);
	MipsCpu.GPR[rd].lo = temp;
	
	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Shift Word Right Logical Variable
To execute a logical right-shift of a word by a variable number of bits
MIPS32
*/
void   SLRV () {	
	_u64 temp;
	
	Extract_rt();
	Extract_rd();
	Extract_rs();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: srlv %s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd], debug_r5900_reg[rt], debug_r5900_reg[rs] );
	
	temp = ( MipsCpu.GPR[rt].lo & 0xFFFFFFFF )  >>  ( MipsCpu.GPR[rs].lo & 0x1F);	
	sign_extended(temp);
	MipsCpu.GPR[rd].lo = temp;
	
	MipsCpu.PC = MipsCpu.PC + 4;
}


/*
Shift Word Right Arithmetic Variable
To execute an arithmetic right-shift of a word by a variable number of bits
MIPS32
*/

void   SRAV () {		
	_u64 temp;
	_u64 boucle;
	
	Extract_rt();
	Extract_rd();
	Extract_rs();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: srav %s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd], debug_r5900_reg[rt], debug_r5900_reg[rs] );
	
	if (MipsCpu.GPR[rt].lo & 0x0000000080000000LL) {
		//printf("SRA %d: %8.8llx => ",( MipsCpu.GPR[rs].lo & 0x1F),MipsCpu.GPR[rt].lo);
		temp = MipsCpu.GPR[rt].lo & 0xFFFFFFFF;
		for ( boucle = 0 ; boucle < ( MipsCpu.GPR[rs].lo & 0x1F) ; boucle++ ) temp = ( temp >> 1 ) | 0x80000000 ;
		//printf("%8.8llx\n",temp);
	} else 	temp =  MipsCpu.GPR[rt].lo >>  ( MipsCpu.GPR[rs].lo & 0x1F)  ; 
	
	
	sign_extended(temp);
	MipsCpu.GPR[rd].lo = temp;
	
	MipsCpu.PC = MipsCpu.PC + 4;
}


/*
Jump Register
To execute a branch to an instruction address in a register
MIPS32
*/
void   JR   () {
	_u64	branch_target;
	Extract_rs();

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: jr %s with %s=%8llx", MipsCpu.PC, debug_r5900_reg[rs],debug_r5900_reg[rs] , MipsCpu.GPR[rs].lo );

	if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );

	branch_target = MipsCpu.GPR[rs].lo;
	MipsCpu.PC = MipsCpu.PC + 4;
	MipsUpdateCpu();
	MipsCpu.PC =  branch_target;
}

/*
Jump and Link Register
To execute a procedure call to an instruction address in a register
MIPS32
*/
void   JALR () {
	_u64	branch_target;
	Extract_rs();
	Extract_rd();
	
	branch_target = MipsCpu.GPR[rs].lo;
	MipsCpu.GPR[rd].lo = MipsCpu.PC + 8;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: jalr %s,%s with %s=%8llx", MipsCpu.PC, debug_r5900_reg[rd],debug_r5900_reg[rs] ,debug_r5900_reg[rs], MipsCpu.GPR[rs].lo );
	if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );

	MipsCpu.PC = MipsCpu.PC + 4;
	MipsUpdateCpu();

	MipsCpu.PC =  branch_target;
}

/*
Move Conditional on Zero
To conditionally move a GPR after testing a GPR value
MIPS32
*/
void   MOVZ() {		
	Extract_rs();
	Extract_rt();
	Extract_rd();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: movz %s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd],debug_r5900_reg[rs] ,debug_r5900_reg[rt] );
	
	if(  MipsCpu.GPR[rt].lo == 0 ) MipsCpu.GPR[rd].lo = MipsCpu.GPR[rs].lo ;
		
	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Move Conditional on Not Zero
To conditionally move a GPR after testing a GPR value
MIPS32
*/
void   MOVN() {
	Extract_rs();
	Extract_rt();
	Extract_rd();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: movn %s,%s,%s ", MipsCpu.PC, debug_r5900_reg[rd],debug_r5900_reg[rs] ,debug_r5900_reg[rt] );
	
	if(  MipsCpu.GPR[rt].lo != 0 ) MipsCpu.GPR[rd].lo = MipsCpu.GPR[rs].lo ;
		
	MipsCpu.PC = MipsCpu.PC + 4;
	
	
}

/*
System Call
To cause a System Call exception
MIPS32
*/
void   SYSCALL() {

//g	if ( debug_print_syscall == 1 )	printf("%8llx : %s ( %x h) \n", MipsCpu.PC , debug_syscall[(MipsCpu.GPR[3].lo & 0xFF)],(_u8) (MipsCpu.GPR[3].lo & 0xFF));
/*
	printf("	v0: %16.16llx\n", MipsCpu.GPR[2].lo );
	printf("	v1: %16.16llx\n", MipsCpu.GPR[3].lo );	
	printf("	a0: %16.16llx\n", MipsCpu.GPR[4].lo );
	printf("	a1: %16.16llx\n", MipsCpu.GPR[5].lo );	
	printf("	a2: %16.16llx\n", MipsCpu.GPR[6].lo );	
	getchar();
*/
	syscall_fct[(MipsCpu.GPR[3].lo & 0xFF)]();

}

/*
Breakpoint
To cause a Breakpoint exception
MIPS32
*/
void   BREAK () {	MipsCpu.halt = 1;}


/*
Synchronize Shared Memory
To order loads and stores.
MIPS32
*/
void   SYNC  () {
	sprintf(debugTempText,"%8.8llx: SYNC", MipsCpu.PC);
	MipsCpu.PC = MipsCpu.PC + 4;
}


/*
Move From HI Register
To copy the special purpose HI register to a GPR
MIPS32
*/
void   MFHI   () {
	
	Extract_rd();
	
	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mhi %s", MipsCpu.PC, debug_r5900_reg[rd]  );
	
	MipsCpu.GPR[rd].lo = MipsCpu.GPR[CPU_REG_HI].lo;

	MipsCpu.PC = MipsCpu.PC + 4;
}

void   MTHI  () {	MipsCpu.halt = 1;}

/*
Move From LO Register
To copy the special purpose LO register to a GPR
MIPS32
*/
void   MFLO  () {		Extract_rd();
	
	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mflo %s", MipsCpu.PC, debug_r5900_reg[rd]  );
	
	MipsCpu.GPR[rd].lo = MipsCpu.GPR[CPU_REG_LO].lo;

	MipsCpu.PC = MipsCpu.PC + 4;
}

void   MTLO  () {	MipsCpu.halt = 1;}
void   DSLLV () {	MipsCpu.halt = 1;}
void   DSRLV () {	MipsCpu.halt = 1;}
void   DSRAV () {	MipsCpu.halt = 1;}

/*
Multiply Word
To multiply 32-bit signed integers
MIPS32
*/
void   MULT     () {
	_s64 temp;
	
	Extract_rs();
	Extract_rt();
	Extract_rd();			

	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mult %s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd] , debug_r5900_reg[rs],debug_r5900_reg[rt] );

//	MipsCpu.GPR[CPU_REG_LO].lo = ( MipsCpu.GPR[rs].lo & 0xFFFFFFFF ) * ( MipsCpu.GPR[rt].lo & 0xFFFFFFFF );
//	MipsCpu.GPR[CPU_REG_HI].lo = ( (MipsCpu.GPR[rs].lo >> 32 ) & 0xFFFFFFFF ) * ( (MipsCpu.GPR[rt].lo >> 32 )& 0xFFFFFFFF );
	
	temp = (_s32)(MipsCpu.GPR[rs].lo  ) * (_s32)( MipsCpu.GPR[rt].lo  );
	
	MipsCpu.GPR[CPU_REG_LO].lo = temp & 0xFFFFFFFF;
	MipsCpu.GPR[CPU_REG_HI].lo = ( temp & 0xFFFFFFFF00000000 ) >> 32 ;

	sign_extended(MipsCpu.GPR[CPU_REG_LO].lo);
	sign_extended(MipsCpu.GPR[CPU_REG_HI].lo);



	MipsCpu.GPR[rd].lo = MipsCpu.GPR[CPU_REG_LO].lo ;
	
	MipsCpu.PC = MipsCpu.PC + 4;
}

void   MULTU() {	MipsCpu.halt = 1;}

/*
Divide Word
To divide a 32-bit signed integers
MIPS32
*/
void   DIV  () {		
	ldiv_t result_div;
	Extract_rs();
	Extract_rt();
	
	if ( debug_op == 1 )  	sprintf(debugTempText,"%8.8llx: div %s,%s", MipsCpu.PC, debug_r5900_reg[rs],debug_r5900_reg[rt]);

	
	result_div = ldiv( (_s32) (MipsCpu.GPR[rs].lo & 0xFFFFFFFF )  , (_s32) (MipsCpu.GPR[rt].lo & 0xFFFFFFFF )  );
	MipsCpu.GPR[CPU_REG_LO].lo = result_div.quot;
	MipsCpu.GPR[CPU_REG_HI].lo = result_div.rem;

	sign_extended( MipsCpu.GPR[CPU_REG_LO].lo);
	sign_extended( MipsCpu.GPR[CPU_REG_HI].lo);

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Divide Unsigned Word
To divide a 32-bit unsigned integers
MIPS32
*/

void   DIVU () {

	
	ldiv_t result_div;
	Extract_rs();
	Extract_rt();
	
	if ( debug_op == 1 )  	sprintf(debugTempText,"%8.8llx: divu %s,%s", MipsCpu.PC, debug_r5900_reg[rs],debug_r5900_reg[rt]);

	
	result_div = ldiv(  (MipsCpu.GPR[rs].lo & 0xFFFFFFFF )  ,  (MipsCpu.GPR[rt].lo & 0xFFFFFFFF )  );
	MipsCpu.GPR[CPU_REG_LO].lo = result_div.quot;
	MipsCpu.GPR[CPU_REG_HI].lo = result_div.rem;

	sign_extended( MipsCpu.GPR[CPU_REG_LO].lo);
	sign_extended( MipsCpu.GPR[CPU_REG_HI].lo);

	MipsCpu.PC = MipsCpu.PC + 4;
}

void   DMULT  () {	MipsCpu.halt = 1;}

/*
Doubleword Multiply Unsigned
To multiply 64-bit unsigned integers
MIPS64

Following code is part of arithmetic.cc of SimuAlpha Version 2.2a
Copyright (c) 1999,2000,2001, Kenji KISE
*/

void   DMULTU () {	
	_u64 ah, al, bh, bl;
	_u64 x, y, z,rh,rl;
	
	int carry = 0;
	
	Extract_rs();
	Extract_rt();
	Extract_rd();			
	
	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: dmultu	%s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd] , debug_r5900_reg[rs],debug_r5900_reg[rt] );
	
	ah = MipsCpu.GPR[rs].lo >> 32;
	al = MipsCpu.GPR[rs].lo & 0xFFFFFFFF;
	
	bh = MipsCpu.GPR[rt].lo >> 32;
	bl = MipsCpu.GPR[rt].lo & 0xFFFFFFFF;
	
	
	x = al*bl;
    y = (al*bh << 32);
    z = x + y;
    if(z<x || z<y) carry ++;
    x = z;
    y = (ah*bl << 32);
    z = x + y;
    if(z<x || z<y) carry ++;
    
    rl = z;
     
    rh = ah*bh+ (al*bh >> 32) + (ah*bl >> 32) + carry;
	
	MipsCpu.GPR[CPU_REG_LO].lo = rl;
	MipsCpu.GPR[CPU_REG_HI].lo = rh;

	sign_extended(MipsCpu.GPR[CPU_REG_LO].lo);
	sign_extended(MipsCpu.GPR[CPU_REG_HI].lo);



	MipsCpu.GPR[rd].lo = MipsCpu.GPR[CPU_REG_LO].lo ;
	
	MipsCpu.PC = MipsCpu.PC + 4;
	
	
}


void   DDIV   () {	MipsCpu.halt = 1;}
void   DDIVU  () {	MipsCpu.halt = 1;}


/*
Add Word
To add 32-bit integers. If an overflow occurs, then trap.
MIPS32
*/
void   ADD     () {
	_u64 temp;
	_u64 check;
	
	Extract_rt(); 
	Extract_rs();
	Extract_rd();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: add	%s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd],debug_r5900_reg[rt],debug_r5900_reg[rs] );
	
	
	temp = MipsCpu.GPR[rs].lo + MipsCpu.GPR[rt].lo;
	
	check =  ( temp >> 30 ) & 3;
	
	if ( (check == 0) ||(check == 3) ) {
		sign_extended(temp);
		MipsCpu.GPR[rd].lo = temp;
	} else {
		printf("add Integer Overflow\n" );
	}
	

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Add Unsigned Word
To add 32-bit integers
MIPS32
*/
void   ADDU    () {
	_u64 temp;
	
	Extract_rt(); 
	Extract_rs();
	Extract_rd();
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: addu %s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd],debug_r5900_reg[rt],debug_r5900_reg[rs] );
	
	
	temp = MipsCpu.GPR[rs].lo + MipsCpu.GPR[rt].lo;
	sign_extended(temp);
	MipsCpu.GPR[rd].lo = temp;

	MipsCpu.PC = MipsCpu.PC + 4;

}


/*
Subtract Word
To subtract 32-bit integers. If overflow occurs, then trap
MIPS32
*/
void   SUB () {
	_u64 temp;
	_u64 check;
	
	Extract_rt(); 
	Extract_rs();
	Extract_rd();
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sub	%s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd],debug_r5900_reg[rs],debug_r5900_reg[rt] );
	
	temp = MipsCpu.GPR[rs].lo - MipsCpu.GPR[rt].lo;	
	check =  ( temp >> 30 ) & 3;
	
	if ( (check == 0) ||(check == 3) ) {
		sign_extended(temp);
		MipsCpu.GPR[rd].lo = temp;
	} else {
		printf("sub Integer Overflow\n" );
	}
	

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Subtract Unsigned Word 
To subtract 32-bit integers
MIPS32
*/
void   SUBU   () {
	_u64 temp;
	
	Extract_rt(); 
	Extract_rs();
	Extract_rd();

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: subu %s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd],debug_r5900_reg[rs],debug_r5900_reg[rt] );
	
	
	temp = MipsCpu.GPR[rs].lo - MipsCpu.GPR[rt].lo;
	sign_extended(temp);
	MipsCpu.GPR[rd].lo = temp;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
And
To do a bitwise logical AND
MIPS32
*/
void   AND () {
	Extract_rt(); 
	Extract_rs();
	Extract_rd();
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: and	%s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd],debug_r5900_reg[rs],debug_r5900_reg[rt] );

	MipsCpu.GPR[rd].lo = MipsCpu.GPR[rs].lo & MipsCpu.GPR[rt].lo;


	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Or
To do a bitwise logical OR
MIPS32
*/
void   OR () {
	Extract_rt(); 
	Extract_rs();
	Extract_rd();
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: or %s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd],debug_r5900_reg[rs],debug_r5900_reg[rt] );

	MipsCpu.GPR[rd].lo = MipsCpu.GPR[rs].lo | MipsCpu.GPR[rt].lo;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Xor
To do a bitwise logical XOR
MIPS32
*/
void   XOR  () {
	Extract_rt(); 
	Extract_rs();
	Extract_rd();
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: or %s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd],debug_r5900_reg[rs],debug_r5900_reg[rt] );

	MipsCpu.GPR[rd].lo = MipsCpu.GPR[rs].lo ^ MipsCpu.GPR[rt].lo;

	MipsCpu.PC = MipsCpu.PC + 4;
}


void   NOR   () {	MipsCpu.halt = 1;}

/*
Set on Less Than
To record the result of a less-than comparison
MIPS32
*/
void   SLT () {
	_u64	temp1;
	_u64	temp2;
	
	

	
	Extract_rs();
	Extract_rt();
	Extract_rd();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: slt %s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd], debug_r5900_reg[rs], debug_r5900_reg[rt]);
	
	temp1 = MipsCpu.GPR[rs].lo;
	temp2 = MipsCpu.GPR[rt].lo;
	
	sign_extended(temp1);
	sign_extended(temp2);


    if  ((_s64) temp1 < (_s64) temp2 ) MipsCpu.GPR[rd].lo = 1; 
    else 	MipsCpu.GPR[rd].lo = 0; 

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Set on Less Than Unsigned
To record the result of an unsigned less-than comparison
MIPS32
*/
void   SLTU   () {
	Extract_rs();
	Extract_rt();
	Extract_rd();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sltu %s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd], debug_r5900_reg[rs], debug_r5900_reg[rt]);
 
    if  ( 	MipsCpu.GPR[rs].lo < 	MipsCpu.GPR[rt].lo ) 	MipsCpu.GPR[rd].lo = 1; 
    else 	MipsCpu.GPR[rd].lo = 0; 

	MipsCpu.PC = MipsCpu.PC + 4;
}

void   DADD  () {	MipsCpu.halt = 1;}


/*
Doubleword Add Unsigned
To add 64-bit integers
MIPS64
*/
void   DADDU  () {
	
	_u64 temp;
	
	
	Extract_rs();
	Extract_rt();
	Extract_rd();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: daddu %s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd], debug_r5900_reg[rs], debug_r5900_reg[rt]);
	
	temp = MipsCpu.GPR[rs].lo + MipsCpu.GPR[rt].lo;
	sign_extended(temp);
	MipsCpu.GPR[rd].lo = temp;
	
	MipsCpu.PC = MipsCpu.PC + 4;
}

void   DSUB  () {	MipsCpu.halt = 1;}


/*
Doubleword Subtract Unsigned
To subtract 64-bit integers
MIPS64
*/
void   DSUBU() {
	_u64 temp;
	_u64 check;
	
	Extract_rt(); 
	Extract_rs();
	Extract_rd();
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsubu %s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd],debug_r5900_reg[rs],debug_r5900_reg[rt] );
	
	
	temp = MipsCpu.GPR[rs].lo - MipsCpu.GPR[rt].lo;
	check =  ( temp >> 62 ) & 3;
	if ( (check == 0) ||(check == 3) ) {
		//sign_extended(temp);
		MipsCpu.GPR[rd].lo = temp;
	} else {
		printf("dsubu Integer Overflow\n" );
		//sign_extended(temp);
		MipsCpu.GPR[rd].lo = temp;
	}
	
	MipsCpu.PC = MipsCpu.PC + 4;

}

void   TGE   () {	MipsCpu.halt = 1;}
void   TGEU  () {	MipsCpu.halt = 1;}
void   TLT   () {	MipsCpu.halt = 1;}
void   TLTU  () {	MipsCpu.halt = 1;}
void   TEQ   () {	MipsCpu.halt = 1;}
void   TNE   () {	MipsCpu.halt = 1;}


/*
Doubleword Shift Left Logical Variable
To execute a left-shift of a doubleword by a variable number of bits
MIPS64
*/
void   DSLL     () {
	Extract_rt();
	Extract_rd();
	Extract_sa();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsll %s,%s,%4x", MipsCpu.PC, debug_r5900_reg[rd], debug_r5900_reg[rt],sa );
	
	MipsCpu.GPR[rd].lo = MipsCpu.GPR[rt].lo << sa ;

	MipsCpu.PC = MipsCpu.PC + 4;
}


/*
Doubleword Shift Right Logical DSRL
To execute a logical right-shift of a doubleword by a fixed amount 0 to 31 bits
MIPS64
*/
void   DSRL  () {
	Extract_rt();
	Extract_rd();
	Extract_sa();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsrl %s,%s,%4x", MipsCpu.PC, debug_r5900_reg[rd], debug_r5900_reg[rt],sa );
	
	MipsCpu.GPR[rd].lo = MipsCpu.GPR[rt].lo >> sa ;

	MipsCpu.PC = MipsCpu.PC + 4;
}

void   DSRA   () {	MipsCpu.halt = 1;}


/*
Doubleword Shift Left Logical Plus 32
To execute a left-shift of a doubleword by a fixed amount 32 to 63 bits
MIPS64
*/
void   DSLL32  () {
	Extract_rt();
	Extract_rd();
	Extract_sa();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsll32 %s,%s,%4x", MipsCpu.PC, debug_r5900_reg[rd], debug_r5900_reg[rt],sa );
	
	MipsCpu.GPR[rd].lo = MipsCpu.GPR[rt].lo << ( sa + 32 ) ;

	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Doubleword Shift Right Logical Plus 32
To execute a logical right-shift of a doubleword by a fixed amount 32 to 63 bits
MIPS64
*/
void   DSRL32   () {
	Extract_rt();
	Extract_rd();
	Extract_sa();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsrl32 %s,%s,%4x", MipsCpu.PC, debug_r5900_reg[rd], debug_r5900_reg[rt],sa );
	
	MipsCpu.GPR[rd].lo = MipsCpu.GPR[rt].lo >> ( sa + 32 ) ;
	
	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Doubleword Shift Right Arithmetic Plus 32
To execute an arithmetic right-shift of a doubleword by a fixed amount 32 to 63 bits
MIPS64
*/
void   DSRA32() {
	_u64 temp;
	
	Extract_rt();
	Extract_rd();
	Extract_sa();
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsra32 %s,%s,%4x", MipsCpu.PC, debug_r5900_reg[rd], debug_r5900_reg[rt],sa );

	if (MipsCpu.GPR[rt].lo & 0x80000000000000000LL) { 
		
		
		
		printf("NEED dsra32 Adjust\n");
	} else {
	
		temp = MipsCpu.GPR[rt].lo >> ( sa + 32 ) ;
		MipsCpu.GPR[rd].lo = temp;
	}
	
	MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Branch on Less Than Zero Likely
To test a GPR then do a PC-relative conditional branch; execute the delay slot only if the branch is taken.
MIPS32
*/
void	BLTZ() {
	_u64	target_branch;
	
	Extract_rs();
	Extract_offset();
	
	offset = offset << 2;
	sign_extendedh(offset);
	target_branch = MipsCpu.PC + 4 +  offset ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bltz %s,%4llx", MipsCpu.PC, debug_r5900_reg[rs], target_branch );

	if ((_s64)( MipsCpu.GPR[rs].lo) < 0 ) {
			if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
			MipsCpu.PC = MipsCpu.PC + 4;
			MipsUpdateCpu();
			MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Branch on Greater Than or Equal to Zero
To test a GPR then do a PC-relative conditional branch
MIPS32
*/
void      BGEZ() {
	_u64	target_branch;
	
	Extract_rs();
	Extract_offset();
	
	offset = offset << 2;
	sign_extendedh(offset);
	target_branch = MipsCpu.PC + 4 +  offset ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bgez %s,%4llx", MipsCpu.PC, debug_r5900_reg[rs], target_branch );

	if ( (_s64)(MipsCpu.GPR[rs].lo) >= 0 ) {
			if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
			MipsCpu.PC = MipsCpu.PC + 4;
			MipsUpdateCpu();
			MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 4;
}

/*
Branch on Less Than Zero Likely


*/
void      BLTZL  () {		
	_u64	target_branch;
	
	Extract_rs();

	Extract_offset();
	
	offset = offset << 2;
	sign_extendedh(offset);
	target_branch = MipsCpu.PC + 4 +  offset ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bltzl %s,%4llx", MipsCpu.PC,debug_r5900_reg[rs],target_branch );

	if ( (_s64)MipsCpu.GPR[rs].lo < 0 ) {
			if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
			MipsCpu.PC = MipsCpu.PC + 4;
			MipsUpdateCpu();
			MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8;
}



/*
Branch on Greater Than or Equal to Zero Likely
To test a GPR then do a PC-relative conditional branch; execute the delay slot only if the branch is taken.
MIPS32
*/
void      BGEZL  () {		
	_u64	target_branch;
	
	Extract_rs();

	Extract_offset();
	
	offset = offset << 2;
	sign_extendedh(offset);
	target_branch = MipsCpu.PC + 4 +  offset ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bgezl %s,%4llx", MipsCpu.PC,debug_r5900_reg[rs],target_branch );

	if ( (_s64)MipsCpu.GPR[rs].lo >= 0 ) {
			if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
			MipsCpu.PC = MipsCpu.PC + 4;
			MipsUpdateCpu();
			MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8;
}


void      TGEI   () {	MipsCpu.halt = 1;}
void      TGEIU  () {	MipsCpu.halt = 1;}
void      TLTI   () {	MipsCpu.halt = 1;}
void      TLTIU  () {	MipsCpu.halt = 1;}
void      TEQI   () {	MipsCpu.halt = 1;}
void      TNEI   () {	MipsCpu.halt = 1;}
void      BLTZAL () {	MipsCpu.halt = 1;}
void      BGEZAL () {	MipsCpu.halt = 1;}
void      BLTZALL() {	MipsCpu.halt = 1;}
void      BGEZALL() {	MipsCpu.halt = 1;}



/*
Move from Coprocessor 0
To move the contents of a coprocessor 0 register to a general register.
MIPS32
*/
void   	MFC0  () {
	_u64 data;

	Extract_rt();
	Extract_rd();
	Extract_sel();


	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mfc0 %s,%s,%d", MipsCpu.PC, debug_r5900_reg[rt],debug_r5900_cop0_reg[rd],sel );
	
	data = MipsCpu.CPR0[rd][sel];
	sign_extended(data);
	
	MipsCpu.GPR[rt].lo = data ;
	MipsCpu.PC = MipsCpu.PC + 4;
	
}


void   	DMFC0 () {	MipsCpu.halt = 1;}
void   	MTC0  () {	MipsCpu.halt = 1;}
void   	DMTC0 () {	MipsCpu.halt = 1;}


void	C0    () {	cop0c0_opc[(MipsCpu.opcode&0x3F)](); }

void	TLBR  () {	MipsCpu.halt = 1;}
void	TLBWI () {	MipsCpu.halt = 1;}
void	TLBWR () {	MipsCpu.halt = 1;}
void	TLBP  () {	MipsCpu.halt = 1;}	
void	ERET  () {	MipsCpu.halt = 1;}
void	DERET () {	MipsCpu.halt = 1;}
void 	WAIT  () {	MipsCpu.halt = 1;}


void 	EI() {
	MipsCpu.CPR0[12][0]	=	MipsCpu.CPR0[12][0]  | 0x00010000;
	
	MipsCpu.PC = 	MipsCpu.PC + 4;
}


void 	DI() {
	MipsCpu.CPR0[12][0]	=	MipsCpu.CPR0[12][0]  & 0xFFFEFFFF;
	
	MipsCpu.PC = 	MipsCpu.PC + 4;
}


/*********************************************************************************************/

void SYSCALL_PrintDebugInfo() {

	printf("%8.8llx : %s 0x%x %d \n", MipsCpu.PC , debug_syscall[(MipsCpu.GPR[CPU_REG_V1].lo & 0xFF)],(_u8) (MipsCpu.GPR[CPU_REG_V1].lo & 0xFF),(char) (MipsCpu.GPR[CPU_REG_V1].lo & 0xFF));
	
}


void SYSCALL_NI() {

	printf("%8.8llx : Syscall_NI ( %x h) s8:%d u8:%d\n",MipsCpu.PC ,(_u8) (MipsCpu.GPR[CPU_REG_V1].lo & 0xFF),(char) (MipsCpu.GPR[CPU_REG_V1].lo & 0xFF),(_u8) (MipsCpu.GPR[CPU_REG_V1].lo & 0xFF));
	
	MipsCpu.halt = 1;
}




void SYSCALL_RFU000_FullReset( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();
	MipsCpu.halt = 1; 
} 




/*
ResetEE	ENTRY_1	8000ad68

*/
void SYSCALL_ResetEE( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();
	MipsCpu.halt = 1; 
}  





/*
SetGsCrt 02h
8000a060
Need Implementation
Do nothing
*/
void SYSCALL_SetGsCrt( ) { 

	if ( debug_print_syscall == 1 )	{
		SYSCALL_PrintDebugInfo();
		printf("|???\n");
	}
	

	MipsCpu.PC = 	MipsCpu.PC + 4;	
} 




void SYSCALL_RFU003( ) { 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 
 
void SYSCALL_Exit( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 


/*
RFU005	ENTRY_5	80002880
*/ 
void SYSCALL_RFU005( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}


/*
LoadPS2Exe	ENTRY_6	800055a0

*/  
void SYSCALL_LoadPS2Exe( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 


/*
ExecPS2	ENTRY_7	80002f80


*/ 
void SYSCALL_ExecPS2( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
RFU008	ENTRY_8

*/  
void SYSCALL_RFU008( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}  

/*
RFU009	ENTRY_9	80005b28
*/
void SYSCALL_RFU009( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}  

/*
AddSbusIntcHandler	ENTRY_10	80001e78

*/
void SYSCALL_AddSbusIntcHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
RemoveSbusIntcHandler	ENTRY_11	80001ed0


*/  
void SYSCALL_RemoveSbusIntcHandler( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
Interrupt2Iop	ENTRY_12	80001f70

*/  
void SYSCALL_Interrupt2Iop( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}


/*
SetVTLBRefillHandler	ENTRY_13	800009c0


*/  
void SYSCALL_SetVTLBRefillHandler( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
SetVCommonHandler	ENTRY_14	80000a00


*/  
void SYSCALL_SetVCommonHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}  

/*
SetVInterruptHandler	ENTRY_15	80000a40


*/
void SYSCALL_SetVInterruptHandler( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}  

/*
AddIntcHandler	ENTRY_16	80001a30


*/
void SYSCALL_AddIntcHandler( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}  

/*
RemoveIntcHandler	ENTRY_17	80001a70


*/
void SYSCALL_RemoveIntcHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}  


/*
AddDmacHandler	ENTRY_18	80001c78

a0: channel number
a1: Address handler
*/
void SYSCALL_AddDmacHandler( ) {
	
	if ( debug_print_syscall == 1 )	{
		SYSCALL_PrintDebugInfo();
		printf("|channel: %2lld, Address: 0x%8.8llx\n",MipsCpu.GPR[CPU_REG_A0].lo,MipsCpu.GPR[CPU_REG_A1].lo);
	}
	Ps2Bios.DmaH[ ( MipsCpu.GPR[CPU_REG_A0].lo ) ].Add = MipsCpu.GPR[CPU_REG_A1].lo ;
	
	
	//MipsCpu.halt = 1;
	MipsCpu.PC = MipsCpu.PC + 4;
} 

/*
RemoveDmacHandler	ENTRY_19	80001cb8


*/


void SYSCALL_RemoveDmacHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}   

/*
_EnableIntc	ENTRY_20	800008c0


*/

void SYSCALL__EnableIntc( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}   

/*

_DisableIntc	ENTRY_21	80000900


*/
void SYSCALL__DisableIntc( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}   

/*
_EnableDmac	ENTRY_22	80000940

a0 : Channel Number

void 80000940 ( ) {
  
		v0 = Mem [ 0xB000E010 ] ;
		at = 0x1000 << a0 ;
		v0 = v0 & at
		
		if ( v0 != 0 ) {
			v0 = 0 ;
			return
		}
		
		Mem [ 0xB000E010 ] = at ;
		v0 = 1 
}




*/
void SYSCALL__EnableDmac( )  					{

	_u128 t_v0;
	_u128 t_at;
	

	if ( debug_print_syscall == 1 )	{
		SYSCALL_PrintDebugInfo();
		printf("|Channel: %lld\n",MipsCpu.GPR[CPU_REG_A0].lo);
	}
	
	MipsMemoryRead ( &t_v0 , DATA_DW ,  0x1000E010  ,0 );
	DmaUpdate();
	t_at.lo = 0x10000 << MipsCpu.GPR[CPU_REG_A0].lo ;
	t_v0.lo = t_v0.lo & t_at.lo ;
	
	if ( t_v0.lo != 0 ) MipsCpu.GPR[CPU_REG_V0].lo = 0 ;
	else {
		
		MipsMemoryWrite ( t_at  , DATA_DW ,  0x1000E010  ,0 ) ;
		DmaUpdate();
		MipsCpu.GPR[CPU_REG_V0].lo =  1;
	}
		
	MipsCpu.PC = MipsCpu.PC + 4;

} 


/*
_DisableDmac	ENTRY_23	80000980


*/
void SYSCALL__DisableDmac( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
_SetAlarm	ENTRY_24	800022d8


*/ 
void SYSCALL__SetAlarm( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 

/*
_ReleaseAlarm	ENTRY_25	80002570

*/
void SYSCALL__ReleaseAlarm( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 


/*
CreateThread	ENTRY_32	80003c50


*/
void SYSCALL_CreateThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
DeleteThread	ENTRY_33	80002fc0


*/ 
void SYSCALL_DeleteThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
StartThread	ENTRY_34	80003040


*/ 
void SYSCALL_StartThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}


/*
ExitThread	ENTRY_35	800030c0


*/ 
void SYSCALL_ExitThread( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 

/*
ExitDeleteThread	ENTRY_36	80003100


*/
void SYSCALL_ExitDeleteThread( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
TerminateThread	ENTRY_37	80003140
*/
 
void SYSCALL_TerminateThread( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 

/*
DisableDispatchThread	ENTRY_39	80004228

*/
void SYSCALL_DisableDispatchThread( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 

/*
EnableDispatchThread	ENTRY_40	80004258

*/
void SYSCALL_EnableDispatchThread( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_ChangeThreadPriority( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 




void SYSCALL_RotateThreadReadyQueue( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 




void SYSCALL_ReleaseWaitThread( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 




void SYSCALL_GetThreadId( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_ReferThreadStatus( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_SleepThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_WakeupThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_CancelWakeupThread( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_SuspendThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_ResumeThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_JoinThread( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 

/*

*/
void	SYSCALL_RFU060() {
	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();		
	MipsCpu.GPR[CPU_REG_V0].lo = MipsCpu.GPR[CPU_REG_SP].lo ;				
	MipsCpu.PC = MipsCpu.PC + 4;

}

/*


*/
void	SYSCALL_RFU061() {
	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();			
	MipsCpu.GPR[CPU_REG_V0].lo =	MipsCpu.GPR[CPU_REG_A0].lo + 	MipsCpu.GPR[CPU_REG_A1].lo ;	
	MipsCpu.PC = MipsCpu.PC + 4;

}




void SYSCALL_EndOfHeap( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

  



void SYSCALL_RFU063( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}  


/*
a1: Number of sema
*/
void SYSCALL_CreateSema( ) 						{ 
		
	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();
	if ( Ps2Bios.NbSema != 0 ) {
		MipsCpu.GPR[CPU_REG_V0].lo =  0;

	
	} else {	
	
		Ps2Bios.Sema[ Ps2Bios.NbSema ] = 0;
		Ps2Bios.NbSema++;
	
		MipsCpu.GPR[CPU_REG_V0].lo =  1;
	}	

	MipsCpu.PC = MipsCpu.PC + 4;	
} 



void SYSCALL_DeleteSema( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}  



void SYSCALL_SignalSema( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_WaitSema( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_PollSema( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_ReferSemaStatus( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU073( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_SetOsdConfigParam( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_GetOsdConfigParam( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_GetGsHParam( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}  



void SYSCALL_GetGsVParam( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_SetGsHParam( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_SetGsVParam( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU080_CreateEventFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU081_DeleteEventFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU082_SetEventFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU083_iSetEventFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU084_ClearEventFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU085_iClearEventFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU086_WaitEvnetFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU087_PollEvnetFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU088_iPollEvnetFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU089_ReferEventFlagStatus( ) 	{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU091( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_EnableIntcHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_DisableIntcHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_EnableDmacHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_DisableDmacHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_KSeg0( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_EnableCache( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}  



void SYSCALL_DisableCache( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_GetCop0( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   

/*
FlushCache 0x64 100
Do Nothing
*/
void SYSCALL_FlushCache( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	
	MipsCpu.PC = MipsCpu.PC + 4;
}   



void SYSCALL_CpuConfig( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_sceSifStopDma( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_SetCPUTimerHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_SetCPUTimer( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_ForceRead( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_ForceWrite( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   

/*

*/
void SYSCALL_GsGetIMR( ) 						{ 
	if ( debug_print_syscall == 1 )	{ 
		SYSCALL_PrintDebugInfo();
		printf("|???\n");
	}	
	
	MipsCpu.PC = MipsCpu.PC + 4;
} 

/*
GsPutIMR
Write A0 to 0x12001010
V0 = A0
*/
void SYSCALL_GsPutIMR( ) 						{ 
	if ( debug_print_syscall == 1 )	{
		SYSCALL_PrintDebugInfo();		
		printf("|A0	= %llx\n",MipsCpu.GPR[CPU_REG_A0].lo);
	}
	
	
	MipsCpu.GPR[CPU_REG_V0].lo = MipsCpu.GPR[CPU_REG_A0].lo;
	MipsMemoryWrite ( MipsCpu.GPR[CPU_REG_A0] , DATA_DW ,  0x12001010  ,0 ) ;			// (12001010 : GS_IMR) = A0
	
	
	
	MipsCpu.PC = MipsCpu.PC + 4;
} 




void SYSCALL_SetPgifHandler( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}    



void SYSCALL_SetVSyncFlag( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}    



void SYSCALL_RFU116( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}    


void SYSCALL_print( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}    



void SYSCALL_sceSifDmaStat( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}    




void SYSCALL_sceSifSetDma( ) 					{ 

	if ( debug_print_syscall == 1 )	{
		SYSCALL_PrintDebugInfo();
		printf("|A0	= %llx, A1 = %llx \n",MipsCpu.GPR[CPU_REG_A0].lo,MipsCpu.GPR[CPU_REG_A1].lo);
	
	}
	
	
	if ( Ps2Bios.DmaH[5].Running == 0 ) Ps2Bios.Sif[ 0 ].Enable = MipsCpu.GPR[CPU_REG_A1].lo;

	//MipsCpu.halt = 1; 
	MipsCpu.PC = MipsCpu.PC + 4;
} 


/*
#---------------------------------------------------------------------------------------
#ENTRY_120
#isceSifSetDChain

void 800062A0( ) {
 		
		v1 =  0xB000C000 :
		v0 =  0xB000C020 ;
					
		Mem [ v0 ] = 0 ;
		Mem [ v1 ] = 0 ;
		
		a0 = 0x184;		
		
		sp = sp  - 0x10 ;
		
		Mem [ v1 ] = a0;
		v0 = Mem [ v1 ] ;
		
		Mem [ sp ] = v0 ;
		
		sp = sp + 0x10 ;
		
}
#---------------------------------------------------------------------------------------



*/
void SYSCALL_sceSifSetDChain( )  				{	
	_u128 temp;
	
	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();		
	
	temp.lo = 0;

	MipsMemoryWrite ( temp , DATA_DW ,  0x1000c020  ,0 ) ;
	DmaUpdate();

	MipsCpu.GPR[CPU_REG_A0].lo  = 0x00000184;
	MipsMemoryWrite ( MipsCpu.GPR[CPU_REG_A0] , DATA_DW ,  0x1000c000  ,0 ) ;
	DmaUpdate();
	
	MipsMemoryRead ( &MipsCpu.GPR[CPU_REG_V0] , DATA_DW , 0x1000c000, 0 );

	printf("|%llx \n",MipsCpu.GPR[CPU_REG_V0].lo);

	
	MipsMemoryWrite ( MipsCpu.GPR[CPU_REG_V0] , DATA_DW ,  MipsCpu.GPR[CPU_REG_SP].lo - 16 ,0 ) ;
	MipsCpu.PC = MipsCpu.PC + 4;
		
} 
 


void SYSCALL_sceSifSetReg( ) 					{ 


	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     

/*
sceSifGetReg	ENTRY_122	80006c18


*/
void SYSCALL_sceSifGetReg( ) 					{
	
	if ( debug_print_syscall == 1 ) {
		SYSCALL_PrintDebugInfo();
		
			
		printf("|a0=%llx\n", MipsCpu.GPR[CPU_REG_A0].lo);
	}
	
	MipsCpu.GPR[CPU_REG_V0].lo = 0x00060000;
	MipsCpu.PC = MipsCpu.PC + 4;
/*
	if ( MipsCpu.GPR[CPU_REG_A0].lo == 2 ) {
		 MipsCpu.GPR[CPU_REG_V1].lo = 0x1000F210;
		 MipsCpu.PC = MipsCpu.PC + 4;
		 return;
		}

	if ( MipsCpu.GPR[CPU_REG_A0].lo == 1 ) {
		 MipsCpu.GPR[CPU_REG_V1].lo = 0x1000F200;
		 MipsCpu.PC = MipsCpu.PC + 4;
		 return;
		}
	
	if ( MipsCpu.GPR[CPU_REG_A0].lo  == 3 ) {
		MipsMemoryRead ( &MipsCpu.GPR[CPU_REG_V0] , DATA_W , 0x1000F220, 0 );	
		MipsCpu.PC = MipsCpu.PC + 4;
		return;
	}
	
	if ( MipsCpu.GPR[CPU_REG_A0].lo == 4 ) {
		MipsMemoryRead ( &MipsCpu.GPR[CPU_REG_V0] , DATA_W , 0x1000F230, 0 );
		MipsCpu.PC = MipsCpu.PC + 4;
		return;
	}
	
	if ( MipsCpu.GPR[CPU_REG_A0].lo >= 0x80000000 ) {
		
		MipsCpu.GPR[CPU_REG_V0].lo = MipsCpu.GPR[CPU_REG_A0].lo & 0x7FFFFFFF;
	
		if ( MipsCpu.GPR[CPU_REG_V0].lo > 0x20 ) {
			MipsCpu.GPR[CPU_REG_V0].lo = 0;
			MipsCpu.PC = MipsCpu.PC + 4;
			return;
		}

		MipsCpu.GPR[CPU_REG_V0].lo = MipsCpu.GPR[CPU_REG_V0].lo << 2 ;
		MipsCpu.GPR[CPU_REG_AT].lo = 0x8001E080 + MipsCpu.GPR[CPU_REG_V0].lo;
		MipsMemoryRead ( &MipsCpu.GPR[CPU_REG_V0] , DATA_W , MipsCpu.GPR[CPU_REG_AT].lo, 0 );
		MipsCpu.PC = MipsCpu.PC + 4;
		
	}
	else {
		MipsCpu.GPR[CPU_REG_V0].lo = 0;
		
	}
*/	
	 
	
} 
 



void SYSCALL_ExecOSD( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_Deci2Call( ) 						{ 

	if ( debug_print_syscall == 1 ) SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_PSMode( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_MachineType( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_GetMemorySize( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_isceSifSetDChain( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_isceSifSetDma( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_isceSifDmaStat( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


/*
GsPutIMR	ENTRY_113	80000d40
12001010	IMR			Interrupt mask

*/
void SYSCALL_iGsPutIMR( ) 						{ 
	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();
	MipsCpu.PC = MipsCpu.PC + 4;
} 


/*
GsGetIMR	ENTRY_112	80000d00



*/
void SYSCALL_iGsGetIMR( ) 						{

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();
	MipsCpu.PC = MipsCpu.PC + 4;
} 



void SYSCALL_iCpuConfig( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iFlushCache( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iGetCop0( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iDisableDmacHandler( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iEnableDmacHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iDisableIntcHandler( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iEnableIntcHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_RFU090_iReferEventFlagStatus( )	{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iReferSemaStatus( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iPollSema( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_iSignalSema( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iResumeThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iSuspendThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iCancelWakeupThread( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iWakeupThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iReferThreadStatus( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iReleaseWaitThread( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iRotateThreadReadyQueue( )			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iChangeThreadPriority( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}    


void SYSCALL_iTerminateThread( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL__iReleaseAlarm( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL__iSetAlarm( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL__iDisableDmac( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
} 


void SYSCALL__iEnableDmac( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL__iDisableIntc( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL__iEnableIntc( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_UKN_1() { 
	
	if ( debug_print_syscall == 1 )	{
		SYSCALL_PrintDebugInfo();	
		printf("|???\n");
	}
	
	//MipsCpu.halt = 1;
	MipsCpu.PC = MipsCpu.PC + 4; 
}    

/*********************************************************************************************************************/

void	MADD( )		{ MipsCpu.halt = 1; } 

		
void	MADDU( )	{ MipsCpu.halt = 1; } 
void	MUL( )		{ MipsCpu.halt = 1; } 
void	MSUB( )		{ MipsCpu.halt = 1; } 
void	MSUBU( )	{ MipsCpu.halt = 1; } 
void	CLZ( )		{ MipsCpu.halt = 1; } 
void	CLO( )		{ MipsCpu.halt = 1; } 
void	DLCZ( ) 	{ MipsCpu.halt = 1; } 
void	DLCO( )		{ MipsCpu.halt = 1; } 
void	SDBBP( )	{ MipsCpu.halt = 1; } 

/*
???


*/
void	MULT1( )	{
	_u64 hi_temp,lo_temp;
	

	Extract_rs();
	Extract_rt();
	Extract_rd();			

	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mult1 %s,%s,%s", MipsCpu.PC, debug_r5900_reg[rd] , debug_r5900_reg[rs],debug_r5900_reg[rt] );

	lo_temp = (( MipsCpu.GPR[rs].lo & 0xFFFFFFFF ) * ( MipsCpu.GPR[rt].lo & 0xFFFFFFFF )) ;
	hi_temp = ( (MipsCpu.GPR[rs].lo >> 32 ) & 0xFFFFFFFF ) * ( (MipsCpu.GPR[rt].lo >> 32 )& 0xFFFFFFFF );
	
	MipsCpu.GPR[CPU_REG_LO].hi = lo_temp;
	MipsCpu.GPR[CPU_REG_HI].hi = hi_temp;
	
	MipsCpu.GPR[rd].lo = lo_temp ;
	
	
	MipsCpu.PC = MipsCpu.PC + 4;
	
}



/*
Move Word From Floating Point
To copy a word from an FPU (CP1) general register to a GPR
MIPS32
*/
void	MFC1()	{ 
	
	//_s64 temp;
	
	Extract_fs();
	Extract_rt();
	
	
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: mfc1 %s,%s", MipsCpu.PC, debug_r5900_reg[rt],debug_r5900_cop1_reg[fs] );
	
	//temp = MipsCpu.FPR1_float[fs];
	
	MipsCpu.GPR[rt].lo = (_s64)MipsCpu.FPR1_float[fs] & 0xFFFFFFFF;		//Create a  the integer with a float

	
	//printf("%f %llx %llx \n",MipsCpu.FPR1_float[fs] ,temp, MipsCpu.GPR[rt].lo);
	
	MipsCpu.PC = MipsCpu.PC + 4;
	
	
}


void	DMFC1()	{ MipsCpu.halt = 1; }	
void	CFC1()	{ MipsCpu.halt = 1; }			

/*
Move Word to Floating Point
To copy a word from a GPR to an FPU (CP1) general register
MIPS32
*/
void	MTC1()	{ 
	Extract_fs();
	Extract_rt();
	
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: mtc1 %s,%s", MipsCpu.PC, debug_r5900_reg[rt],debug_r5900_cop1_reg[fs] );

	
	MipsCpu.FPR1[fs].lo = MipsCpu.GPR[rt].lo;						//Copy the val to the reg
	MipsCpu.FPR1_float[fs] = *(MipsCpu.FPR1_p_float[fs]);			//Make the conversion to the Emu Val
	
	//printf("%f \n",MipsCpu.FPR1_float[fs]);
	
	MipsCpu.PC = MipsCpu.PC + 4;	
	
	
	
}	

void	DMTC1()	{ MipsCpu.halt = 1; }	
void	CTC1()	{ MipsCpu.halt = 1; }	


void	BC1()	{ cop1_bc1_opc[(MipsCpu.opcode>>16)&0x3](); }	 		

void	COP1_S(){ cop1_s_opc[MipsCpu.opcode&0x3F](); };

void	COP1_D(){ cop1_d_opc[MipsCpu.opcode&0x3F](); }	

void	COP1_W(){ cop1_W_L_opc[MipsCpu.opcode&0x3F]();	}

void	COP1_L(){ MipsCpu.halt = 1; }




void	BC1F()	{ 	
	_u64	target_branch;
	_u8		condition;
	
	Extract_ccb();
	Extract_offset();
	
	offset = offset << 2;
	sign_extendedh(offset);
	target_branch = MipsCpu.PC + 4 +  offset ;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bc1f %x,%4llx", MipsCpu.PC, ccb,target_branch );


	if ( ccb == 0 ) {

		condition = ( MipsCpu.CP1[CP1_FCSR] & 0x00800000 ) >> 23;
	
		if ( condition == 0 ) {
				if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
				MipsCpu.PC = MipsCpu.PC + 4;
				MipsUpdateCpu();
				MipsCpu.PC = target_branch;
		}
		else MipsCpu.PC = MipsCpu.PC + 4; 
	
		
	} else {
		printf("BC1F : cc not implemented\n");
		MipsCpu.halt = 1;
	}
	
	
	//printf("BC1TL : condition = %x \n",condition); 
} 
/*
Branch on FP True
To test an FP condition code and do a PC-relative conditional branch
MIPS32
*/
void	BC1T()	{ 	
	_u64	target_branch;
	_u8		condition;
	
	Extract_ccb();
	Extract_offset();
	
	offset = offset << 2;
	sign_extendedh(offset);
	target_branch = MipsCpu.PC + 4 +  offset ;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bc1t %x,%4llx", MipsCpu.PC, ccb,target_branch );


	if ( ccb == 0 ) {

		condition = ( MipsCpu.CP1[CP1_FCSR] & 0x00800000 ) >> 23;
	
		if ( condition == 1 ) {
				if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
				MipsCpu.PC = MipsCpu.PC + 4;
				MipsUpdateCpu();
				MipsCpu.PC = target_branch;
		}
		else MipsCpu.PC = MipsCpu.PC + 4; 
	
		
	} else {
		printf("BC1T : cc not implemented\n");
		MipsCpu.halt = 1;
	}
	
	
	//printf("BC1TL : condition = %x \n",condition); 
} 
void	BC1FL()	{ MipsCpu.halt = 1; }

/*
Branch on FP True Likely
To test an FP condition code and do a PC-relative conditional branch; execute the instruction in the delay slot only if
the branch is taken.
MIPS32
*/
void	BC1TL()	{ 	
	
	_u64	target_branch;
	_u8		condition;
	
	Extract_ccb();
	Extract_offset();
	
	offset = offset << 2;
	sign_extendedh(offset);
	target_branch = MipsCpu.PC + 4 +  offset ;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bc1tl %x,%4llx", MipsCpu.PC, ccb,target_branch );


	if ( ccb == 0 ) {

		condition = ( MipsCpu.CP1[CP1_FCSR] & 0xFF7FFFFF ) >> 23;
	
		if ( condition == 1 ) {
				if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
				MipsCpu.PC = MipsCpu.PC + 4;
				MipsUpdateCpu();
				MipsCpu.PC = target_branch;
		}
		else MipsCpu.PC = MipsCpu.PC + 8; 
	
		
	} else {
		printf("BC1TL : cc not implemented\n");
		MipsCpu.halt = 1;
	}
	
	//printf("BC1TL : condition = %x \n",condition);

	
	//MipsCpu.halt = 1;
}

/*
Floating Point Add
To add floating point values
MIPS32
*/
void	ADD_S()	{ 


	Extract_fd();
	Extract_fs();
	Extract_ft();
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: add_s %s,%s,%s", MipsCpu.PC, debug_r5900_cop1_reg[fd],debug_r5900_cop1_reg[fs] ,debug_r5900_cop1_reg[ft]);
	
	MipsCpu.FPR1_float[fd] = (float)(MipsCpu.FPR1_float[fs] + MipsCpu.FPR1_float[ft] ) ;
	MipsCpu.FPR1[fd].lo = *(MipsCpu.FPR1_p_u64[fd]);				//Make the conversion to the CPU Val
	
//	printf("%f %llx\n",MipsCpu.FPR1_float[fd],MipsCpu.FPR1[fd].lo);
	

	MipsCpu.PC = MipsCpu.PC + 4;

	
} 		

/*
Floating Point Subtract
To subtract FP values
MIPS32
*/
void	SUB_S()	{ 

	Extract_fd();
	Extract_fs();
	Extract_ft();
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: sub_s %s,%s,%s\n", MipsCpu.PC, debug_r5900_cop1_reg[fd],debug_r5900_cop1_reg[fs] ,debug_r5900_cop1_reg[ft]);
	
	MipsCpu.FPR1_float[fd] = (float)(MipsCpu.FPR1_float[fs] - MipsCpu.FPR1_float[ft] );
	MipsCpu.FPR1[fd].lo = *(MipsCpu.FPR1_p_u64[fd]);				//Make the conversion to the CPU Val
	
//	printf("%f %llx\n",MipsCpu.FPR1_float[fd],MipsCpu.FPR1[fd].lo);
	

	MipsCpu.PC = MipsCpu.PC + 4;
	
} 		


/*
Floating Point Multiply
To multiply FP values
MIPS32
*/
void	MUL_S()	{ 

	Extract_fd();
	Extract_fs();
	Extract_ft();
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: mul_s %s,%s,%s", MipsCpu.PC, debug_r5900_cop1_reg[fd],debug_r5900_cop1_reg[fs] ,debug_r5900_cop1_reg[ft]);
	
	MipsCpu.FPR1_float[fd] = (float)(MipsCpu.FPR1_float[fs] * MipsCpu.FPR1_float[ft] );
	MipsCpu.FPR1[fd].lo = *(MipsCpu.FPR1_p_u64[fd]);				//Make the conversion to the CPU Val
	
	//printf("%f %llx\n",MipsCpu.FPR1_float[fd],MipsCpu.FPR1[fd].lo);
	

	MipsCpu.PC = MipsCpu.PC + 4;
	
} 		

/*
Floating Point Divide
To divide FP values
MIPS32
*/
void	DIV_S()	{ 
	Extract_fd();
	Extract_fs();
	Extract_ft();
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: div_s %s,%s,%s", MipsCpu.PC, debug_r5900_cop1_reg[fd],debug_r5900_cop1_reg[fs] ,debug_r5900_cop1_reg[ft]);
	
//	if ( MipsCpu.FPR1_float[ft] != 0.0 ) {
	
		MipsCpu.FPR1_float[fd] = (float)(MipsCpu.FPR1_float[fs] /(float) MipsCpu.FPR1_float[ft]) ;
		MipsCpu.FPR1[fd].lo = *(MipsCpu.FPR1_p_u64[fd]);				//Make the conversion to the CPU Val
//	}	
	
	//	printf("%f %llx\n",MipsCpu.FPR1_float[fd],MipsCpu.FPR1[fd].lo);
	

	MipsCpu.PC = MipsCpu.PC + 4;

} 		

void	SQRT_S(){ MipsCpu.halt = 1; } 	
void	ABS_S()	{ MipsCpu.halt = 1; } 		

/*
Floating Point Move
To move an FP value between FPRs
MIPS32
*/
void	MOV_S()	{ 
	
	Extract_fs();
	Extract_fd();
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: mov_s %s,%s", MipsCpu.PC, debug_r5900_cop1_reg[fd],debug_r5900_cop1_reg[fs] );
	
	MipsCpu.FPR1_float[fd] = MipsCpu.FPR1_float[fs] ;		//Copy the emu val
	MipsCpu.FPR1[fd].lo = MipsCpu.FPR1[fs].lo ;				//Copy the cpu val

	MipsCpu.PC = MipsCpu.PC + 4;
} 		

/*
Floating Point Negate
To negate an FP value
MIPS32
*/
void	NEG_S()		{ 	
	Extract_fs();
	Extract_fd();
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: mov_s %s,%s", MipsCpu.PC, debug_r5900_cop1_reg[fd],debug_r5900_cop1_reg[fs] );
	
	MipsCpu.FPR1_float[fd] = -MipsCpu.FPR1_float[fs] ;		//Copy the emu val
	MipsCpu.FPR1[fd].lo = *(MipsCpu.FPR1_p_u64[fd]);		//Make the conversion to the CPU Val

	MipsCpu.PC = MipsCpu.PC + 4;
}

void	ROUND_L_S()	{ MipsCpu.halt = 1; }	
void	TRUNC_L_S()	{ MipsCpu.halt = 1; }	
void	CEIL_L_S()	{ MipsCpu.halt = 1; }	
void	FLOOR_L_S()	{ MipsCpu.halt = 1; }	
void	ROUND_W_S()	{ MipsCpu.halt = 1; }	
void	TRUNC_W_S()	{ MipsCpu.halt = 1; }	
void	CEIL_W_S()	{ MipsCpu.halt = 1; }	
void	FLOOR_W_S()	{ MipsCpu.halt = 1; }
void	MOVCF_S()	{ MipsCpu.halt = 1; }	
void	MOVZ_S()	{ MipsCpu.halt = 1; }		
void	MOVN_S()	{ MipsCpu.halt = 1; }		
void	RECIP_S()	{ MipsCpu.halt = 1; }  	
void	RSQRT_S()	{ MipsCpu.halt = 1; }  	
void	CVT_D_S()	{ MipsCpu.halt = 1; }		

/*
Floating Point Convert to Word Fixed Point
To convert an FP value to 32-bit fixed point
MIPS32
*/
void	CVT_W_S()	{ 
		
	Extract_fs();
	Extract_fd();
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: cvt_w_s	%s, %s", MipsCpu.PC, debug_r5900_cop1_reg[fd],debug_r5900_cop1_reg[fs] );
	
	MipsCpu.FPR1_float[fd] = (int)MipsCpu.FPR1_float[fs] ;		//Copy the emu val
	MipsCpu.FPR1[fd].lo = *(MipsCpu.FPR1_p_u64[fd]);			//Make the conversion to the CPU Val

	MipsCpu.PC = MipsCpu.PC + 4;

}	

void	CVT_L_S()	{ MipsCpu.halt = 1; }	
void	C_F_S()		{ MipsCpu.halt = 1; } 		
void	C_UN_S()	{ MipsCpu.halt = 1; }		

/*
Floating Point Compare

MIPS32
*/
void	C_EQ_S()	{ 
	Extract_fs();
	Extract_ft();
	Extract_cc();
	

	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: c_eq_s %x,%s,%s", MipsCpu.PC, cc, debug_r5900_cop1_reg[fs],debug_r5900_cop1_reg[ft] );

	if ( cc == 0 ) {
		if ( MipsCpu.FPR1_float[fs] == MipsCpu.FPR1_float[ft] ) {
			
			MipsCpu.CP1[CP1_FCSR] = MipsCpu.CP1[CP1_FCSR] | 0x00800000;
				
		} else {
			
			MipsCpu.CP1[CP1_FCSR] = MipsCpu.CP1[CP1_FCSR] & 0x17FFFFF;
		
		}
	} else {
		printf("C_EQ_S : cc not implemented\n");
		MipsCpu.halt = 1;
	}
	
	//printf("%16.16lld : CP1_FCSR : %f %f => %llx \n", MipsCpu.Nb_instructions, MipsCpu.FPR1_float[fs],MipsCpu.FPR1_float[ft],MipsCpu.CP1[CP1_FCSR]);
	


	MipsCpu.PC = MipsCpu.PC + 4;

}		

void	C_UEQ_S()	{ MipsCpu.halt = 1; }	


/*
Condition Field : 2..0 = 4
4 = 100 bin
*/
void	C_OLT_S()	{ 	
	Extract_fs();
	Extract_ft();
	Extract_cc();
	

	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: c_olt_s %x,%s,%s", MipsCpu.PC, cc, debug_r5900_cop1_reg[fs],debug_r5900_cop1_reg[ft] );

	if ( cc == 0 ) {
		if ( MipsCpu.FPR1_float[fs] < MipsCpu.FPR1_float[ft] ) {
			
			MipsCpu.CP1[CP1_FCSR] = MipsCpu.CP1[CP1_FCSR] | 0x4000000;
				
		} else {
			
			MipsCpu.CP1[CP1_FCSR] = MipsCpu.CP1[CP1_FCSR] & 0x17FFFFF;
		
		}
	} else {
		printf("C_OLT_S : cc not implemented\n");
		MipsCpu.halt = 1;
	}
	
	//printf("%16.16lld : CP1_FCSR : %f %f => %llx \n", MipsCpu.Nb_instructions, MipsCpu.FPR1_float[fs],MipsCpu.FPR1_float[ft],MipsCpu.CP1[CP1_FCSR]);
	


	MipsCpu.PC = MipsCpu.PC + 4; 
}	

void	C_ULT_S()	{ MipsCpu.halt = 1; }	
void	C_OLE_S()	{ MipsCpu.halt = 1; }	
void	C_ULE_S()	{ MipsCpu.halt = 1; }
void	C_SF_S()	{ MipsCpu.halt = 1; }		
void	C_NGLE_S()	{ MipsCpu.halt = 1; } 	
void	C_SEQ_S()	{ MipsCpu.halt = 1; }	
void	C_NGL_S()	{ MipsCpu.halt = 1; }	
void	C_LT_S()	{ MipsCpu.halt = 1; }		
void	C_NGE_S()	{ MipsCpu.halt = 1; }	
void	C_LE_S()	{ MipsCpu.halt = 1; }		
void	C_NGT_S()	{ MipsCpu.halt = 1; }



void	ADD_D()		{ MipsCpu.halt = 1; }		
void	SUB_D()		{ MipsCpu.halt = 1; }		
void	MUL_D()		{ MipsCpu.halt = 1; }		
void	DIV_D()		{ MipsCpu.halt = 1; }		
void	SQRT_D()	{ MipsCpu.halt = 1; }		
void	ABS_D()		{ MipsCpu.halt = 1; }		
void	MOV_D()		{ MipsCpu.halt = 1; }		
void	NEG_D()		{ MipsCpu.halt = 1; }
void	ROUND_L_D()	{ MipsCpu.halt = 1; }	
void	TRUNC_L_D()	{ MipsCpu.halt = 1; }	
void	CEIL_L_D()	{ MipsCpu.halt = 1; }	
void	FLOOR_L_D()	{ MipsCpu.halt = 1; }	
void	ROUND_W_D()	{ MipsCpu.halt = 1; }	
void	TRUNC_W_D()	{ MipsCpu.halt = 1; }	
void	CEIL_W_D()	{ MipsCpu.halt = 1; }	
void	FLOOR_W_D()	{ MipsCpu.halt = 1; }
void	MOVCF_D()	{ MipsCpu.halt = 1; }	
void	MOVZ_D()	{ MipsCpu.halt = 1; }		
void	MOVN_D()	{ MipsCpu.halt = 1; }				
void	RECIP_D()	{ MipsCpu.halt = 1; }	
void	RSQRT_D()	{ MipsCpu.halt = 1; }
void	CVT_S_D()	{ MipsCpu.halt = 1; }	
void	CVT_W_D()	{ MipsCpu.halt = 1; }	
void	CVT_L_D()	{ MipsCpu.halt = 1; }	
void	C_F_D()		{ MipsCpu.halt = 1; }		
void	C_UN_D()	{ MipsCpu.halt = 1; }		
void	C_EQ_D()	{ MipsCpu.halt = 1; }		
void	C_UEQ_D()	{ MipsCpu.halt = 1; }	
void	C_OLT_D()	{ MipsCpu.halt = 1; }	
void	C_ULT_D()	{ MipsCpu.halt = 1; }	
void	C_OLE_D()	{ MipsCpu.halt = 1; }	
void	C_ULE_D()	{ MipsCpu.halt = 1; }
void	C_SF_D()	{ MipsCpu.halt = 1; }		
void	C_NGLE_D()	{ MipsCpu.halt = 1; }	
void	C_SEQ_D()	{ MipsCpu.halt = 1; }	
void	C_NGL_D()	{ MipsCpu.halt = 1; }	
void	C_LT_D()	{ MipsCpu.halt = 1; }		
void	C_NGE_D()	{ MipsCpu.halt = 1; }	
void	C_LE_D()	{ MipsCpu.halt = 1; }		
void	C_NGT_D()	{ MipsCpu.halt = 1; }

/*
Floating Point Convert to Word Fixed Point
To convert an FP value to 32-bit fixed point
MIPS32
*/
void	CVT_S()		{ 
	Extract_fs();
	Extract_fd();
	

	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: cvt_w_s	%s,%s", MipsCpu.PC, debug_r5900_cop1_reg[fs],debug_r5900_cop1_reg[fd] );
	
	
	MipsCpu.FPR1_float[fd] = (float)MipsCpu.FPR1[fs].lo;		//Create a  the float with an integer

	//printf("%f \n",MipsCpu.FPR1_float[fd]);
	
	MipsCpu.PC = MipsCpu.PC + 4;
	
	
	
	}
		
void	CVT_D()		{ MipsCpu.halt = 1; }			


void	MOVF_fmt()	{ MipsCpu.halt = 1; } 
void	MOVT_fmt()	{ MipsCpu.halt = 1; }	

